@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 
 # main.py --- Main module of flo-check-homework
-# Copyright (c) 2011, 2012, 2013 Florent Rougon
+# Copyright (c) 2011, 2012, 2013, 2014  Florent Rougon
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -32,7 +32,7 @@ from . import fch_util, conjugations, image_widgets
 
 
 progname = os.path.basename(sys.argv[0])
-from . import version as progversion
+from . import __version__ as progversion
 
 usage = """Usage: %(progname)s [OPTION ...] [--] PROGRAM [ARGUMENT ...]
 Check specific skills before launching a program.
@@ -259,38 +259,42 @@ class HomeWorkCheckApp(QtGui.QApplication):
         if not os.path.isdir(cacheDir):
             os.makedirs(cacheDir)
 
-        lockFile = os.path.join(cacheDir, "%s.pid" % progname)
-        lockFileDisplayName = os.path.join(cacheDirDisplayName, "%s.pid" % progname)
+        lockFile = os.path.join(cacheDir, "{0}.pid".format(progname))
+        lockFileDisplayName = os.path.join(cacheDirDisplayName,
+                                           "{0}.pid".format(progname))
         try:
             # Genuine file locking APIs are a real mess:
             #   1) The fcntl module isn't available on all platforms, in
             #      particular it is not available on Windows.
-            #   2) fcntl.flock has a number of issues, one of which being that one
-            #      cannot use it to obtain the PID of the process holding an
-            #      exclusive lock on the file we are trying to lock (if any).
-            #   3) fcntl.fcntl could be nice (the C interface is manageable) but
-            #      the Python interface requires one to pass a byte string that is
-            #      a valid struct flock, and this seems to be impossible to do in
-            #      any portable way, even with Ctypes (I think), since for
-            #      instance the type of some fields (e.g., off_t vs. off64_t) is
-            #      determined at compile time at least on Linux, via a #ifdef
-            #      test. Moreover, the order of the fields is unspecified and only
-            #      known to the C compiler (and there may be holes, etc.).
+            #   2) fcntl.flock has a number of issues, one of which being that
+            #      one cannot use it to obtain the PID of the process holding
+            #      an exclusive lock on the file we are trying to lock (if
+            #      any).
+            #   3) fcntl.fcntl could be nice (the C interface is manageable)
+            #      but the Python interface requires one to pass a byte string
+            #      that is a valid struct flock, and this seems to be
+            #      impossible to do in any portable way, even with Ctypes (I
+            #      think), since for instance the type of some fields (e.g.,
+            #      off_t vs. off64_t) is determined at compile time at least on
+            #      Linux, via a #ifdef test. Moreover, the order of the fields
+            #      is unspecified and only known to the C compiler (and there
+            #      may be holes, etc.).
             #   4) fcntl.lockf, in spite of its silly name (since its interface
-            #      uses the constants of flock and not of the POSIX lockf function
-            #      and it actually is nothing else than an interface to fcntl(2)
-            #      locking), could be useful but it doesn't allow access to the
-            #      PID of the locking process in case one can't set a lock on a
-            #      portion of a file, although the underlying fcntl(2) system call
-            #      does provide this information. What a pity...
-            #   5) The lockfile.py module by Skip Montanaro could be interesting but
-            #      isn't part of the Python standard library and seems to have
-            #      outstanding issues for quite some time already (as of Dec
-            #      2012).
+            #      uses the constants of flock and not of the POSIX lockf
+            #      function and it actually is nothing else than an interface
+            #      to fcntl(2) locking), could be useful but it doesn't allow
+            #      access to the PID of the locking process in case one can't
+            #      set a lock on a portion of a file, although the underlying
+            #      fcntl(2) system call does provide this information. What a
+            #      pity...
+            #   5) The lockfile.py module by Skip Montanaro could be
+            #      interesting but isn't part of the Python standard library
+            #      and seems to have outstanding issues for quite some time
+            #      already (as of Dec 2012).
             #
             # For all these reasons, we'll use a simple lock file and write the
-            # PID to this lock file, followed by \n to let readers know when they
-            # have read the whole PID and not only part of its decimal
+            # PID to this lock file, followed by \n to let readers know when
+            # they have read the whole PID and not only part of its decimal
             # representation.
             lockFileFD = os.open(lockFile, os.O_WRONLY | os.O_CREAT | os.O_EXCL,
                                  0o666)
@@ -304,17 +308,18 @@ class HomeWorkCheckApp(QtGui.QApplication):
 
         if otherProcessHasLock:
             time.sleep(1)
-            # We know the PID has been fully written when we encounter os.linesep.
+            # We know the PID has been fully written when we encounter
+            # os.linesep.
             try:
                 with open(lockFile, mode="r", encoding="utf-8",
                           newline=os.linesep) as f:
                     prevSize = 0
                     while True:
                         # We use stat(2) to determine when the file grows and
-                        # sleep at regular intervals. This is not the most elegant
-                        # algorithm ever written but is nevertheless reliable.
-                        # inotify would be more elegant but is Linux-specific (so
-                        # far).
+                        # sleep at regular intervals. This is not the most
+                        # elegant algorithm ever written but is nevertheless
+                        # reliable. inotify would be more elegant but is
+                        # Linux-specific (so far).
                         size = os.stat(lockFile).st_size
                         if size != prevSize:
                             f.seek(0, os.SEEK_SET)
@@ -991,6 +996,8 @@ class MainWindow(QtGui.QMainWindow):
         self.quitTimer.setSingleShot(True)
         self.quitTimer.timeout.connect(self.quitTimerTimeout)
 
+        self.magicFormulaAttempts = 0
+
         self.desiredProgramProcess = QtCore.QProcess(self)
         self.desiredProgramProcess.setProcessChannelMode(
             QtCore.QProcess.ForwardedChannels)
@@ -1296,12 +1303,29 @@ class MainWindow(QtGui.QMainWindow):
 
     @QtCore.pyqtSlot()
     def magicFormula(self):
-        r = random.randint(4, 99)
+        maxAttempts = 3
+
+        if self.magicFormulaAttempts >= maxAttempts:
+            msgBox = QtGui.QMessageBox()
+            msgBox.setText(self.tr(
+                    "Too many failed attempts for the magic word."))
+            msgBox.setInformativeText(self.tr(
+                    "After {0} failed attempts, the magic wand stops working.")
+                                      .format(maxAttempts))
+            msgBox.setStandardButtons(QtGui.QMessageBox.Ok)
+            msgBox.setIcon(QtGui.QMessageBox.Information)
+            msgBox.exec_()
+            return
+
+        self.magicFormulaAttempts += 1
+
+        R1 = random.randint(4, 99)
+        R2 = random.randint(4, 99)
 
         text, ok = QtGui.QInputDialog.getText(
             self, self.tr("Magic word"),
-            self.tr("I say {0}. Please enter the magic word.").format(
-                r),
+            self.tr("I say {0} and {1}. Please enter the magic word.").format(
+                R1, R2),
             QtGui.QLineEdit.Password)
 
         h = time.localtime().tm_hour
@@ -1312,7 +1336,11 @@ class MainWindow(QtGui.QMainWindow):
             except ValueError:
                 return
 
-            if i == 10*h + int(math.sqrt(r)):
+            r1 = int(math.sqrt(R1))
+            r2 = int(math.sqrt(R2))
+
+            if i == 10*(h+r1) + r2:
+                self.magicFormulaAct.setEnabled(False)
                 self.launchDesiredProgramAct.setEnabled(True)
                 self.allowedToQuit = True
 
@@ -1482,7 +1510,7 @@ app.setWindowIcon(icon)
 # QTranslator objects set up and installed in the application.
 version_blurb1 = translate("app", """Written by Florent Rougon.
 
-Copyright (c) 2011-2013  Florent Rougon""")
+Copyright (c) 2011-2014  Florent Rougon""")
 version_blurb2 = translate("app", """\
 This is free software; see the source for copying conditions.  There is NO \
 warranty; not even for MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.""")
